"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Chart = void 0;
const isEqual_1 = __importDefault(require("lodash-es/isEqual"));
const tooltip_1 = require("../tooltip");
const axis_1 = require("../guide/axis");
const line_1 = require("../guide/line");
const point_1 = require("../guide/point");
const line_2 = require("../geometry/line");
const area_1 = require("../geometry/area");
const point_2 = require("../geometry/point");
const interval_1 = require("../geometry/interval");
const coord_1 = require("../coord");
const data_1 = require("./data");
const layout_1 = require("./layout");
const interaction_1 = require("./interaction");
const animation_1 = require("./animation");
const label_1 = require("../guide/label");
class Chart {
    constructor(options) {
        this.components = [];
        this.elements = [];
        this.layoutManager = new layout_1.LayoutManager();
        this.interactionManager = new interaction_1.InteractionManager();
        this.animationManager = new animation_1.AnimationManager(this);
        this.update(options);
    }
    update(options) {
        var _a, _b, _c, _d, _e, _f, _g;
        if (!this.coord || !(0, isEqual_1.default)((_a = this.options) === null || _a === void 0 ? void 0 : _a.coord, options.coord)) {
            const coordOptions = Object.assign(Object.assign({}, ((_b = options.coord) !== null && _b !== void 0 ? _b : {})), { height: options.style.height, width: options.style.width, origin: { x: 0, y: 0 } });
            this.coord = ((_c = options.coord) === null || _c === void 0 ? void 0 : _c.type) === 'polar' ? new coord_1.PolarCoord(coordOptions) : new coord_1.CartesianCoord(coordOptions);
        }
        if (!(0, isEqual_1.default)((_d = this.options) === null || _d === void 0 ? void 0 : _d.style, options.style)) {
            this.layoutManager.update({
                layoutWidth: options.style.width,
                layoutHeight: options.style.height,
                padding: options.style.padding,
            });
        }
        if (!(0, isEqual_1.default)((_e = this.options) === null || _e === void 0 ? void 0 : _e.data, options.data) || !(0, isEqual_1.default)((_f = this.options) === null || _f === void 0 ? void 0 : _f.scale, options.scale)) {
            this.dataSource = new data_1.DataSource(options.data, options.scale);
        }
        if (!(0, isEqual_1.default)((_g = this.options) === null || _g === void 0 ? void 0 : _g.animation, options.animation)) {
            this.animationManager.update(options.animation);
        }
        this.options = options;
    }
    layout() {
        this.layoutManager.layout(this);
    }
    graphic(cid, layout = true) {
        let newElements = [];
        if (layout) {
            this.layout();
        }
        if (cid) {
            // prevElements = this.elements.filter
            newElements = this.elements.filter(element => element.cid !== cid);
        }
        this.components
            .filter(component => (cid ? component.id === cid : true))
            .forEach(component => {
            const graphicOptions = this.layoutManager.get(component.id);
            let elements = component.graphic(this, graphicOptions);
            //animation
            if (!this.coord.isPolar && component.animatable && this.options.animation) {
                const prevElements = this.elements.filter(element => element.cid === component.id);
                elements = this.animationManager.composeAnimation(elements, prevElements);
            }
            newElements.push(...elements);
        });
        this.elements = newElements;
        this.interactionManager.update({
            interactiveElements: this.elements.filter(element => element.interaction),
        });
        return this.elements;
    }
    getOptions() {
        return this.options;
    }
    getComponentsByType(type) {
        return this.components.filter((component) => component.id.startsWith(String(type)));
    }
    getComponentById(id) {
        return this.components.find(component => component.id === id);
    }
    remove(id) {
        this.components = this.components.filter(component => component.id !== id);
        this.elements = this.elements.filter(element => element.cid !== id);
    }
    axis(options) {
        const axis = new axis_1.Axis(this, options);
        this.components.push(axis);
        return axis;
    }
    tooltip(options) {
        const tooltip = new tooltip_1.Tooltip(this, options);
        this.components.push(tooltip);
        return tooltip;
    }
    guideLine(options) {
        const guideLine = new line_1.GuideLine(this, options);
        this.components.push(guideLine);
        return guideLine;
    }
    guidePoint(options) {
        const guidePoint = new point_1.GuidePoint(this, options);
        this.components.push(guidePoint);
        return guidePoint;
    }
    guideLabel(options) {
        const guideLabel = new label_1.GuideLabel(this, options);
        this.components.push(guideLabel);
        return guideLabel;
    }
    point(options) {
        const point = new point_2.Point(this, options);
        this.components.push(point);
        return point;
    }
    line(options) {
        const line = new line_2.Line(this, options);
        this.components.push(line);
        return line;
    }
    area(options) {
        const area = new area_1.Area(this, options);
        this.components.push(area);
        return area;
    }
    interval(options) {
        const interval = new interval_1.Interval(this, options);
        this.components.push(interval);
        return interval;
    }
}
exports.Chart = Chart;
