"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Interval = void 0;
const svg_1 = require("../utils/svg");
const color_1 = require("../const/color");
const abstract_1 = require("./abstract");
const aesthetic_1 = require("./aesthetic");
const data_1 = require("../utils/data");
const isObject_1 = __importDefault(require("lodash/isObject"));
class Interval extends abstract_1.Geometry {
    constructor(chart, options) {
        super(chart);
        this.type = 'Interval';
        this.animatable = true;
        this.defaultAestheticOptions = {
            position: {},
            shape: {},
            size: { value: [10, 10] },
            color: { value: color_1.DEFAULT_COLORS },
            label: {},
        };
        this.update(options);
    }
    update(options) {
        var _a;
        super.update(options);
        const yDimData = this.chart.dataSource.getField(this.yDim).getData();
        const adjustType = typeof options.adjust === 'object' ? (_a = options.adjust) === null || _a === void 0 ? void 0 : _a.type : options.adjust;
        if (adjustType !== 'stack') {
            this.chart.dataSource
                .getField(this.yDim)
                .getScale()
                .update({ domain: [Math.min(...yDimData.map(Number), 0), Math.max(...yDimData.map(Number), 0)] });
        }
        const xScale = this.chart.dataSource.getField(this.xDim).getScale();
        if (!xScale.getOptions().range) {
            const xCount = this.dataSource.getField(this.xDim).getData().filter(data_1.uniqueFilter).length;
            xScale.update({ range: [1 / (xCount + 1), 1 - 1 / (xCount + 1)] });
        }
        this.options = options;
    }
    getCoordinateWidth(chart, graphicOptions, aesthetic) {
        var _a, _b, _c, _d;
        const groupCount = (_a = this.groupedData) === null || _a === void 0 ? void 0 : _a.length;
        const stackCount = (_c = (_b = this.groupedData) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.length;
        const adjustType = (0, isObject_1.default)(this.options.adjust) ? (_d = this.options.adjust) === null || _d === void 0 ? void 0 : _d.type : this.options.adjust;
        if (chart.coord.isPolar) {
            //针对饼图处理，填满全部空间
            if (stackCount === 1 && adjustType === 'stack') {
                return graphicOptions.width;
            }
            else if (stackCount === 1 && adjustType === 'dodge') {
                return graphicOptions.width / groupCount;
            }
        }
        return aesthetic.size();
    }
    graphic(chart, graphicOptions) {
        const aesthetic = new aesthetic_1.Aesthetic(this.defaultAestheticOptions, this.options, graphicOptions, chart.dataSource);
        const width = this.getCoordinateWidth(chart, graphicOptions, aesthetic);
        const positionData = this.generatePositionData(chart, aesthetic, {
            xCentered: true,
            width: width,
        });
        return positionData
            .map((group, groupIndex) => {
            return group.map((data, index, row) => {
                var _a, _b, _c, _d, _e;
                const { record, points } = data;
                const fillColor = aesthetic.color(Object.assign(Object.assign({}, record), { _group: groupIndex }));
                // custom height computation passed by user
                if (this.options.barHeight) {
                    const customHeight = this.options.barHeight(record);
                    if (customHeight !== undefined) {
                        points[1].y = points[0].y - customHeight;
                    }
                }
                const p = [
                    {
                        x: points[0].x,
                        y: points[0].y,
                    },
                    {
                        x: points[1].x,
                        y: points[1].y,
                    },
                    {
                        x: points[1].x + width,
                        y: points[1].y,
                    },
                    {
                        x: points[0].x + width,
                        y: points[0].y,
                    },
                ];
                const interaction = !((_a = chart.coord) === null || _a === void 0 ? void 0 : _a.isPolar)
                    ? {
                        bound: [
                            row[index - 1] ? (row[index - 1].points[1].x + points[1].x) / 2 : graphicOptions.origin.x,
                            -Infinity,
                            row[index + 1] ? (row[index + 1].points[1].x + points[1].x) / 2 : Infinity,
                            Infinity,
                        ],
                        origin: { x: points[1].x + width / 2, y: points[1].y },
                    }
                    : undefined;
                const strokeWidth = (_c = (_b = this.options.itemStyle) === null || _b === void 0 ? void 0 : _b.borderWidth) !== null && _c !== void 0 ? _c : 0;
                const strokeColor = (_e = (_d = this.options.itemStyle) === null || _d === void 0 ? void 0 : _d.borderColor) !== null && _e !== void 0 ? _e : '#fff';
                return {
                    cid: this.id,
                    eid: `${this.id}-${groupIndex}-${index}`,
                    record,
                    shapes: [
                        {
                            type: 'path',
                            d: this.coordinate(p, chart.coord, true),
                            points: p,
                            strokeWidth,
                            strokeColor,
                            fill: fillColor,
                        },
                    ],
                    state: {
                        active: [
                            {
                                type: 'path',
                                d: this.coordinate(p, chart.coord, true),
                                points: p,
                                strokeWidth,
                                strokeColor,
                                fill: fillColor,
                            },
                        ],
                    },
                    interaction,
                };
            });
        })
            .reduce((res, val) => res.concat(val), []); // flatten
    }
    coordinate(points, coord, seal) {
        if (Array.isArray(points)) {
            return coord.isPolar
                ? (0, svg_1.generateCircularPath)(coord, points.map(p => coord.convertPoint(p, false)))
                : (0, svg_1.generateSVGPath)(points.map(p => coord.convertPoint(p)), seal);
        }
        return coord.convertPoint(points);
    }
}
exports.Interval = Interval;
