"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const merge_1 = __importDefault(require("lodash-es/merge"));
const scale_1 = require("@antv/scale");
const scale_2 = require("../types/scale");
const data_1 = require("../utils/data");
const judgement_1 = require("../utils/judgement");
const linearTick_1 = __importDefault(require("./tickMethods/linearTick"));
const categoryTick_1 = __importDefault(require("./tickMethods/categoryTick"));
const CATEGORY_DEFAULT_COUNT = 5;
class Scale {
    constructor(data, options = {}) {
        this.data = data;
        const { type } = options, scaleOptions = __rest(options, ["type"]);
        if (type) {
            // user specified type
            this.type = type;
        }
        else {
            // identify sale type
            const sampleFieldData = data[0];
            switch (typeof sampleFieldData) {
                case 'undefined': {
                    this.type = scale_2.ScaleType.Identity;
                    break;
                }
                case 'number': {
                    this.type = scale_2.ScaleType.Linear;
                    break;
                }
                case 'string': {
                    if ((0, judgement_1.isDateString)(sampleFieldData)) {
                        this.type = scale_2.ScaleType.Time;
                    }
                    else {
                        this.type = scale_2.ScaleType.Category;
                    }
                }
            }
        }
        // generate default options
        switch (this.type) {
            case scale_2.ScaleType.Linear: {
                const scaleOptions = {
                    domain: [Math.min(...data.map(Number)), Math.max(...data.map(Number))],
                    nice: false,
                    tickMethod: linearTick_1.default,
                };
                this.defaultOptions = scaleOptions;
                break;
            }
            case scale_2.ScaleType.Time: {
                const dateValues = data.map(value => new Date(value).getTime());
                const scaleOptions = {
                    domain: [new Date(Math.min(...dateValues)), new Date(Math.max(...dateValues))],
                };
                this.defaultOptions = scaleOptions;
                break;
            }
            case scale_2.ScaleType.TimeCategory:
            case scale_2.ScaleType.Category: {
                const scaleOptions = {
                    domain: data.filter(data_1.uniqueFilter),
                    tickCount: CATEGORY_DEFAULT_COUNT,
                };
                this.defaultOptions = scaleOptions;
                break;
            }
            default: {
                this.defaultOptions = {};
            }
        }
        // merge options
        this.options = (0, merge_1.default)(this.defaultOptions, scaleOptions);
        // create scale
        switch (this.type) {
            case scale_2.ScaleType.Identity:
                this.scale = new scale_1.Identity(this.options);
                break;
            case scale_2.ScaleType.Linear:
                this.scale = new scale_1.Linear(this.options);
                break;
            case scale_2.ScaleType.Time:
                this.scale = new scale_1.Time(this.options);
                break;
            case scale_2.ScaleType.Category:
            case scale_2.ScaleType.TimeCategory:
                this.scale = new scale_1.Point(this.options);
        }
    }
    getDefaultOptions() {
        return this.defaultOptions;
    }
    getOptions() {
        return this.options;
    }
    clone() {
        return new Scale(this.data, Object.assign({ type: this.type }, this.options));
    }
    map(value) {
        return this.type === scale_2.ScaleType.Time ? this.scale.map(new Date(value)) : this.scale.map(value);
    }
    invert(value) {
        return this.scale.invert(value);
    }
    update(updatedOptions) {
        var _a;
        this.options = (0, merge_1.default)(this.options, updatedOptions);
        (_a = this.scale) === null || _a === void 0 ? void 0 : _a.update(updatedOptions);
        return this;
    }
    derive(updatedOptions) {
        const scale = this.clone();
        scale.update(updatedOptions);
        return scale;
    }
    normalization() {
        return this.derive({ range: [0, 1] });
    }
    /**
     * generate a new scale for position purpose, will extend domain to
     * the range of ticks
     *
     * @param {[number, number]} range
     * @memberof Scale
     */
    position({ range }) {
        var _a, _b, _c, _d, _e;
        const rangeRatio = (_a = this.options.range) !== null && _a !== void 0 ? _a : [0, 1];
        const computedRange = rangeRatio.map(ratio => range[0] + ratio * (range[1] - range[0]));
        if (this.scale instanceof scale_1.Continuous) {
            let current = this;
            // sometimes ticks domain is larger than scale domain, this even happens again after we extends domain here
            // so we should extends until this condition will not happen
            // this is why i use while loop here
            do {
                const ticks = current.getTicks();
                const domain = (_b = current.getDomain()) !== null && _b !== void 0 ? _b : [];
                const extendedDomain = [
                    domain[0] < ticks[0] ? domain[0] : ticks[0],
                    domain[domain.length - 1] > ticks[ticks.length - 1] ? domain[domain.length - 1] : ticks[ticks.length - 1],
                ];
                current = this.derive({ domain: extendedDomain, range: computedRange });
            } while (((_c = current.getDomain()) !== null && _c !== void 0 ? _c : [])[0] > current.getTicks()[0] ||
                ((_d = current.getDomain()) !== null && _d !== void 0 ? _d : [])[((_e = current.getDomain()) !== null && _e !== void 0 ? _e : []).length - 1] <
                    current.getTicks()[current.getTicks().length - 1]);
            return current;
        }
        return this.derive({ range: computedRange });
    }
    getTicks() {
        var _a, _b, _c;
        if (this.scale instanceof scale_1.Continuous) {
            return this.scale.getTicks();
        }
        else if ([scale_2.ScaleType.Category, scale_2.ScaleType.TimeCategory].includes(this.type)) {
            return (0, categoryTick_1.default)((_a = this.scale.getDomain()) !== null && _a !== void 0 ? _a : [], (_b = this.options.tickCount) !== null && _b !== void 0 ? _b : CATEGORY_DEFAULT_COUNT);
        }
        else {
            return (_c = this.scale.getOptions().domain) !== null && _c !== void 0 ? _c : [];
        }
    }
    getDomain() {
        return this.scale.getOptions().domain;
    }
}
exports.default = Scale;
