import isEqual from 'lodash-es/isEqual';
import { Tooltip } from '../tooltip';
import { Axis } from '../guide/axis';
import { GuideLine } from '../guide/line';
import { GuidePoint } from '../guide/point';
import { Line as GeometryLine } from '../geometry/line';
import { Area as GeometryArea } from '../geometry/area';
import { Point as GeometryPoint } from '../geometry/point';
import { Interval as GeometryInterval } from '../geometry/interval';
import { CartesianCoord, PolarCoord } from '../coord';
import { DataSource } from './data';
import { LayoutManager } from './layout';
import { InteractionManager } from './interaction';
import { AnimationManager } from './animation';
import { GuideLabel } from '../guide/label';
export class Chart {
    constructor(options) {
        this.components = [];
        this.elements = [];
        this.layoutManager = new LayoutManager();
        this.interactionManager = new InteractionManager();
        this.animationManager = new AnimationManager(this);
        this.update(options);
    }
    update(options) {
        var _a, _b, _c, _d, _e, _f, _g;
        if (!this.coord || !isEqual((_a = this.options) === null || _a === void 0 ? void 0 : _a.coord, options.coord)) {
            const coordOptions = Object.assign(Object.assign({}, ((_b = options.coord) !== null && _b !== void 0 ? _b : {})), { height: options.style.height, width: options.style.width, origin: { x: 0, y: 0 } });
            this.coord = ((_c = options.coord) === null || _c === void 0 ? void 0 : _c.type) === 'polar' ? new PolarCoord(coordOptions) : new CartesianCoord(coordOptions);
        }
        if (!isEqual((_d = this.options) === null || _d === void 0 ? void 0 : _d.style, options.style)) {
            this.layoutManager.update({
                layoutWidth: options.style.width,
                layoutHeight: options.style.height,
                padding: options.style.padding,
            });
        }
        if (!isEqual((_e = this.options) === null || _e === void 0 ? void 0 : _e.data, options.data) || !isEqual((_f = this.options) === null || _f === void 0 ? void 0 : _f.scale, options.scale)) {
            this.dataSource = new DataSource(options.data, options.scale);
        }
        if (!isEqual((_g = this.options) === null || _g === void 0 ? void 0 : _g.animation, options.animation)) {
            this.animationManager.update(options.animation);
        }
        this.options = options;
    }
    layout() {
        this.layoutManager.layout(this);
    }
    graphic(cid, layout = true) {
        let newElements = [];
        if (layout) {
            this.layout();
        }
        if (cid) {
            // prevElements = this.elements.filter
            newElements = this.elements.filter(element => element.cid !== cid);
        }
        this.components
            .filter(component => (cid ? component.id === cid : true))
            .forEach(component => {
            const graphicOptions = this.layoutManager.get(component.id);
            let elements = component.graphic(this, graphicOptions);
            //animation
            if (!this.coord.isPolar && component.animatable && this.options.animation) {
                const prevElements = this.elements.filter(element => element.cid === component.id);
                elements = this.animationManager.composeAnimation(elements, prevElements);
            }
            newElements.push(...elements);
        });
        this.elements = newElements;
        this.interactionManager.update({
            interactiveElements: this.elements.filter(element => element.interaction),
        });
        return this.elements;
    }
    getOptions() {
        return this.options;
    }
    getComponentsByType(type) {
        return this.components.filter((component) => component.id.startsWith(String(type)));
    }
    getComponentById(id) {
        return this.components.find(component => component.id === id);
    }
    remove(id) {
        this.components = this.components.filter(component => component.id !== id);
        this.elements = this.elements.filter(element => element.cid !== id);
    }
    axis(options) {
        const axis = new Axis(this, options);
        this.components.push(axis);
        return axis;
    }
    tooltip(options) {
        const tooltip = new Tooltip(this, options);
        this.components.push(tooltip);
        return tooltip;
    }
    guideLine(options) {
        const guideLine = new GuideLine(this, options);
        this.components.push(guideLine);
        return guideLine;
    }
    guidePoint(options) {
        const guidePoint = new GuidePoint(this, options);
        this.components.push(guidePoint);
        return guidePoint;
    }
    guideLabel(options) {
        const guideLabel = new GuideLabel(this, options);
        this.components.push(guideLabel);
        return guideLabel;
    }
    point(options) {
        const point = new GeometryPoint(this, options);
        this.components.push(point);
        return point;
    }
    line(options) {
        const line = new GeometryLine(this, options);
        this.components.push(line);
        return line;
    }
    area(options) {
        const area = new GeometryArea(this, options);
        this.components.push(area);
        return area;
    }
    interval(options) {
        const interval = new GeometryInterval(this, options);
        this.components.push(interval);
        return interval;
    }
}
