import merge from 'lodash-es/merge';
import { ScaleType } from '../types/scale';
import { isContinuesScale } from '../utils/scale';
import { measureText } from '../utils/text';
import { LINE_COLOR, TEXT_COLOR } from '../const/color';
import { ComponentType } from '../chart/component';
import { Guide } from './abstract';
import { getAxisXLabelAlign, polarToCartesian } from '../utils/polar';
export class Axis extends Guide {
    constructor(chart, options) {
        super(chart, ComponentType.Axis);
        this.animatable = false;
        this.defaultOptions = {
            double: false,
            label: true,
            grid: false,
            formatter: (value) => value instanceof Date ? `${value.getFullYear()}-${value.getMonth() + 1}-${value.getDate()}` : String(value),
            lineStyle: {
                strokeWidth: 1,
                strokeColor: LINE_COLOR,
            },
            tickLineStyle: {
                length: 5,
                strokeWidth: 1,
                strokeColor: LINE_COLOR,
            },
            gridLineStyle: {
                strokeWidth: 1,
                strokeColor: LINE_COLOR,
            },
            labelStyle: {
                fontSize: 10,
                color: TEXT_COLOR,
                offset: 2,
            },
        };
        this.axisType = 'x';
        this.getAxisLine = (graphicOptions, axisOrigin, coord) => {
            const center = coord.center;
            const isPolar = coord.isPolar;
            const from = this.axisType === 'x'
                ? {
                    x: graphicOptions.origin.x,
                    y: axisOrigin,
                }
                : {
                    x: axisOrigin,
                    y: graphicOptions.origin.y + graphicOptions.height,
                };
            const to = this.axisType === 'x'
                ? {
                    x: graphicOptions.origin.x + graphicOptions.width,
                    y: axisOrigin,
                }
                : {
                    x: axisOrigin,
                    y: graphicOptions.origin.y,
                };
            return Object.assign({ type: 'line', from: isPolar ? polarToCartesian(center, from.y, from.x) : from, to: isPolar ? polarToCartesian(center, from.y, from.x) : to }, this.options.lineStyle);
        };
        this.getTicks = (ticks, axisOrigin, scale, coord) => {
            const center = coord.center;
            const isPolar = coord.isPolar;
            const [, r1] = coord.xRange;
            return ticks.map(value => {
                const from = this.axisType === 'x'
                    ? {
                        x: scale.map(value),
                        y: axisOrigin,
                    }
                    : {
                        x: axisOrigin,
                        y: scale.map(value),
                    };
                const to = this.axisType === 'x'
                    ? {
                        x: from.x,
                        y: from.y + this.options.tickLineStyle.length,
                    }
                    : {
                        x: from.x + this.options.tickLineStyle.length,
                        y: from.y,
                    };
                return Object.assign({ type: 'line', from: isPolar ? polarToCartesian(center, r1, from.x) : from, to: isPolar ? polarToCartesian(center, r1, from.x) : to }, this.options.tickLineStyle);
            });
        };
        this.getLabels = (graphicOptions, ticks, axisOrigin, scale, coord) => {
            const center = coord.center;
            const isPolar = coord.isPolar;
            const [, r1] = coord.yRange;
            return ticks.map(value => {
                var _a, _b, _c;
                const pos = this.axisType === 'x'
                    ? {
                        x: scale.map(value),
                        y: axisOrigin + this.options.tickLineStyle.length + this.options.labelStyle.offset,
                    }
                    : {
                        x: axisOrigin - this.options.labelStyle.offset,
                        y: scale.map(value),
                    };
                const polarPos = this.axisType === 'x'
                    ? {
                        x: scale.map(value),
                        y: r1 + this.options.tickLineStyle.length + this.options.labelStyle.offset,
                    }
                    : {
                        x: axisOrigin,
                        y: scale.map(value),
                    };
                const polarAlign = this.axisType === 'x' && isPolar ? getAxisXLabelAlign(polarPos.x) : null;
                let align = 'right'; // y axis label always aligh right
                if (this.axisType === 'x') {
                    if (pos.x === graphicOptions.origin.x) {
                        align = 'left';
                    }
                    else if (pos.x === graphicOptions.origin.x + graphicOptions.width) {
                        align = 'right';
                    }
                    else {
                        align = 'center';
                    }
                }
                const verticalAlign = this.axisType === 'x' ? 'top' : 'middle';
                return Object.assign({ type: 'text', position: isPolar ? polarToCartesian(center, polarPos.y, polarPos.x) : pos, align: (polarAlign === null || polarAlign === void 0 ? void 0 : polarAlign.align) || align, verticalAlign: (polarAlign === null || polarAlign === void 0 ? void 0 : polarAlign.verticalAlign) || verticalAlign, text: [(_c = (_b = (_a = this.options).formatter) === null || _b === void 0 ? void 0 : _b.call(_a, value)) !== null && _c !== void 0 ? _c : ''] }, this.options.labelStyle);
            });
        };
        this.getGrids = (graphicOptions, ticks, scale, coord) => {
            const center = coord.center;
            const [r0, r1] = coord.yRange;
            const origin = this.axisType === 'x' ? graphicOptions.origin.x : graphicOptions.origin.y + graphicOptions.height;
            return ticks
                .filter(tick => scale.map(tick) !== origin)
                .map(tick => {
                const polarGrid = this.axisType === 'x'
                    ? Object.assign({ type: 'line', from: polarToCartesian(center, r0, scale.map(tick)), to: polarToCartesian(center, r1, scale.map(tick)) }, this.options.gridLineStyle) : Object.assign({ type: 'circle', center: center, radius: scale.map(tick), fill: 'none' }, this.options.gridLineStyle);
                const grid = Object.assign({ type: 'line', from: this.axisType === 'x'
                        ? {
                            x: scale.map(tick),
                            y: graphicOptions.origin.y + graphicOptions.height,
                        }
                        : {
                            x: graphicOptions.origin.x,
                            y: scale.map(tick),
                        }, to: this.axisType === 'x'
                        ? {
                            x: scale.map(tick),
                            y: graphicOptions.origin.y,
                        }
                        : {
                            x: graphicOptions.origin.x + graphicOptions.width,
                            y: scale.map(tick),
                        } }, this.options.gridLineStyle);
                return coord.isPolar ? polarGrid : grid;
            });
        };
        this.update(options);
    }
    update(options) {
        this.options = merge({}, this.defaultOptions, options);
        this.field = options.field;
        this.origin = options.origin;
        const scale = this.chart.dataSource.getField(this.field).getScale();
        if (isContinuesScale(scale) || [ScaleType.Category, ScaleType.TimeCategory].includes(scale.type)) {
            if (this.options.tickCount)
                scale.update({ tickCount: this.options.tickCount });
        }
        if (isContinuesScale(scale)) {
            if (this.options.tickMethod)
                scale.update({ tickMethod: this.options.tickMethod });
        }
    }
    layout(chart) {
        var _a;
        const geometry = chart
            .getComponentsByType(ComponentType.Geometry)
            .find(geometry => geometry.xDim === this.field || geometry.yDim === this.field);
        const scale = chart.dataSource.getField(this.field).getScale().normalization();
        if (!this.options.label || !geometry)
            return [0, 0, 0, 0];
        if (chart.coord.isTransposed) {
            this.axisType = this.field === (geometry === null || geometry === void 0 ? void 0 : geometry.xDim) ? 'y' : 'x';
            this.verticalAxisField = this.axisType === 'x' ? geometry.xDim : geometry.yDim;
        }
        else {
            this.axisType = this.field === (geometry === null || geometry === void 0 ? void 0 : geometry.xDim) ? 'x' : 'y';
            this.verticalAxisField = this.axisType === 'x' ? geometry.yDim : geometry.xDim;
        }
        const labels = scale.getTicks().map(tick => { var _a, _b, _c; return (_c = (_b = (_a = this.options).formatter) === null || _b === void 0 ? void 0 : _b.call(_a, tick)) !== null && _c !== void 0 ? _c : ''; });
        const labelSpace = this.options.label
            ? this.options.labelStyle.offset +
                Math.max(...labels.map(label => (this.axisType === 'x' ? measureText(label).height : measureText(label).width)))
            : 0;
        const labelSpaceHeight = this.options.lineStyle.strokeWidth + this.options.tickLineStyle.length + labelSpace;
        const labelSpaceWidth = this.options.lineStyle.strokeWidth + labelSpace;
        // return this.axisType === 'x' ? [0, 0, labelSpace, 0] : [0, 0, 0, labelSpace]
        if ((_a = chart.coord) === null || _a === void 0 ? void 0 : _a.isPolar) {
            return this.axisType === 'x'
                ? [labelSpaceHeight, labelSpaceWidth, labelSpaceHeight, labelSpaceWidth]
                : [0, 0, 0, 0];
        }
        return this.axisType === 'x' ? [0, 0, labelSpaceHeight, 0] : [0, 0, 0, labelSpaceWidth];
    }
    // TODO: apply transform in Chart
    graphic(chart, graphicOptions) {
        var _a, _b, _c, _d, _e, _f;
        // chart.coord.update()
        const coord = chart.coord;
        const xRange = coord.xRange;
        const yRange = coord.yRange;
        const scale = (_b = (_a = chart.dataSource
            .getField(this.field)) === null || _a === void 0 ? void 0 : _a.getScale()) === null || _b === void 0 ? void 0 : _b.position({
            range: this.axisType === 'x' ? xRange : yRange,
        });
        // calculate position in the other axix
        const verticalScale = (_d = (_c = chart.dataSource
            .getField(this.verticalAxisField)) === null || _c === void 0 ? void 0 : _c.getScale()) === null || _d === void 0 ? void 0 : _d.position({
            range: this.axisType === 'x' ? yRange : xRange,
        });
        if (!scale || !verticalScale)
            return [];
        const ticks = (_e = scale.getTicks()) !== null && _e !== void 0 ? _e : [];
        //TODO: chart.layoutManager.origin?
        const verticalAxis = chart
            .getComponentsByType(ComponentType.Axis)
            .find(axis => axis.field === this.verticalAxisField);
        const axisOrigin = chart.coord.isPolar
            ? verticalScale.map((_f = verticalAxis === null || verticalAxis === void 0 ? void 0 : verticalAxis.origin) !== null && _f !== void 0 ? _f : verticalScale.invert(this.axisType === 'x' ? yRange[0] : xRange[0]))
            : chart.layoutManager.origin[this.axisType === 'x' ? 'y' : 'x'];
        // TODO: double
        return [
            {
                cid: this.id,
                eid: this.id,
                shapes: [
                    this.getAxisLine(graphicOptions, axisOrigin, chart.coord),
                    ...this.getTicks(ticks, axisOrigin, scale, coord),
                    ...(this.options.label ? this.getLabels(graphicOptions, ticks, axisOrigin, scale, coord) : []),
                    ...(this.options.grid ? this.getGrids(graphicOptions, ticks, scale, coord) : []),
                ],
            },
        ];
    }
}
